@@ -61,6 +61,8 @@ int main(int argc, char* argv[])
 
   // Before we do much else, load translations
   // This ensures help messages and debugging info will be translated
+  qDebug() << "Locale: " << QLocale::system().name();
+
   QStringList translationPaths;
   // check environment variable and local paths
   foreach (const QString& variable, QProcess::systemEnvironment()) {
@@ -73,33 +75,14 @@ int main(int argc, char* argv[])
 
   translationPaths << QCoreApplication::applicationDirPath() +
                         "/../share/avogadro/i18n/";
+  translationPaths << QLibraryInfo::location(QLibraryInfo::TranslationsPath);
 
-  // Load Qt translations first
-  qDebug() << "Locale: " << QLocale::system().name();
+  QTranslator qtTranslator;
+  QTranslator qtBaseTranslator;
+  QTranslator avoTranslator;
+  QTranslator avoLibsTranslator;  
+  bool tryLoadingQtTranslations = true;
 
-  bool tryLoadingQtTranslations = false;
-  QTranslator qtTranslator(0);
-  if (qtTranslator.load(
-        QLocale::system(), "qt", "_",
-        QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
-    qDebug() << " translation success";
-    app.installTranslator(&qtTranslator);
-  } else {
-    // Check other paths.
-    tryLoadingQtTranslations = true;
-  }
-
-  QTranslator qtBaseTranslator(0);
-  qDebug() << QLibraryInfo::location(QLibraryInfo::TranslationsPath);
-  if (qtTranslator.load(
-        QLocale::system(), "qtbase", "_",
-        QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
-    qDebug() << " translation success";
-    app.installTranslator(&qtTranslator);
-  }
-
-  // TODO: need to separate avogadrolibs from app
-  QTranslator avoTranslator(0);
   foreach (const QString& translationPath, translationPaths) {
     // We can't find the normal Qt translations (maybe we're in a "bundle"?)
     if (tryLoadingQtTranslations) {
@@ -112,11 +95,15 @@ int main(int argc, char* argv[])
         app.installTranslator(&qtBaseTranslator);
       }
     }
-
-    if (avoTranslator.load(QLocale::system(), "avogadrolibs", "_",
+    if (avoTranslator.load(QLocale::system(), "avogadroapp", "_",
                            translationPath)) {
       app.installTranslator(&avoTranslator);
       qDebug() << "Translation successfully loaded.";
+    }    
+    if (avoLibsTranslator.load(QLocale::system(), "avogadrolibs", "_",
+                           translationPath)) {
+      app.installTranslator(&avoLibsTranslator);
+      qDebug() << "Translation successfully loaded.";
     }
   }
 
