@@ -15,6 +15,7 @@
 ******************************************************************************/
 
 #include "mainwindow.h"
+
 #include "aboutdialog.h"
 #include "avogadroappconfig.h"
 #include "backgroundfileformat.h"
@@ -30,6 +31,7 @@
 #include <avogadro/qtgui/customelementdialog.h>
 #include <avogadro/qtgui/extensionplugin.h>
 #include <avogadro/qtgui/fileformatdialog.h>
+#include <avogadro/qtgui/layermodel.h>
 #include <avogadro/qtgui/molecule.h>
 #include <avogadro/qtgui/moleculemodel.h>
 #include <avogadro/qtgui/multiviewwidget.h>
@@ -205,6 +207,7 @@ using QtGui::ExtensionPlugin;
 using QtGui::ExtensionPluginFactory;
 using QtGui::FileFormatDialog;
 using QtGui::Molecule;
+using QtGui::MultiViewWidget;
 using QtGui::RWMolecule;
 using QtGui::ScenePlugin;
 using QtGui::ScenePluginFactory;
@@ -224,6 +227,8 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
   : m_molecule(nullptr)
   , m_rwMolecule(nullptr)
   , m_moleculeModel(nullptr)
+  , m_layerModel(nullptr)
+  , m_activeScenePlugin(nullptr)
   , m_queuedFilesStarted(false)
   , m_menuBuilder(new MenuBuilder)
   , m_fileReadThread(nullptr)
@@ -327,15 +332,16 @@ void MainWindow::setupInterface()
   setCentralWidget(m_multiViewWidget);
   GLWidget* glWidget = new GLWidget(this);
 
-  // set the background color
-  QColor color = settings.value("backgroundColor", QColor(0,0,0,1.0)).value<QColor>();
+  // set the background color (alpha channel default should be opaque)
+  QColor color =
+    settings.value("backgroundColor", QColor(0, 0, 0, 255)).value<QColor>();
   Vector4ub cColor;
   cColor[0] = static_cast<unsigned char>(color.red());
   cColor[1] = static_cast<unsigned char>(color.green());
   cColor[2] = static_cast<unsigned char>(color.blue());
   cColor[3] = static_cast<unsigned char>(color.alpha());
 
-  Avogadro::Rendering::Scene *scene = &glWidget->renderer().scene();
+  Avogadro::Rendering::Scene* scene = &glWidget->renderer().scene();
   scene->setBackgroundColor(cColor);
 
   m_multiViewWidget->addWidget(glWidget);
@@ -346,19 +352,21 @@ void MainWindow::setupInterface()
   addDockWidget(Qt::LeftDockWidgetArea, m_toolDock);
 
   // Our scene/view dock.
-  QDockWidget* sceneDock = new QDockWidget(tr("Display Types"), this);
-  m_sceneTreeView = new QTreeView(sceneDock);
+  m_sceneDock = new QDockWidget(tr("Display Types"), this);
+  m_sceneDock->setFeatures(m_sceneDock->features() &
+                           ~QDockWidget::AllDockWidgetFeatures);
+
+  m_sceneTreeView = new QTreeView(m_sceneDock);
   m_sceneTreeView->setIndentation(0);
   m_sceneTreeView->setSelectionBehavior(QAbstractItemView::SelectRows);
   m_sceneTreeView->setSelectionMode(QAbstractItemView::SingleSelection);
-  sceneDock->setWidget(m_sceneTreeView);
-  addDockWidget(Qt::LeftDockWidgetArea, sceneDock);
-
+  m_sceneDock->setWidget(m_sceneTreeView);
+  addDockWidget(Qt::LeftDockWidgetArea, m_sceneDock);
   // Our view dock.
   m_viewDock = new QDockWidget(tr("View Configuration"), this);
   addDockWidget(Qt::LeftDockWidgetArea, m_viewDock);
-  // put display types on top of view config
-  tabifyDockWidget(m_viewDock, sceneDock);
+  m_viewDock->setFeatures(m_viewDock->features() &
+                          ~QDockWidget::AllDockWidgetFeatures);
 
   // Our molecule dock.
   QDockWidget* moleculeDock = new QDockWidget(tr("Molecules"), this);
@@ -369,6 +377,17 @@ void MainWindow::setupInterface()
   moleculeDock->setWidget(m_moleculeTreeView);
   addDockWidget(Qt::LeftDockWidgetArea, moleculeDock);
 
+  // Our molecule dock.
+  QDockWidget* layerDock = new QDockWidget(tr("Layers"), this);
+  m_layerTreeView = new QTreeView(layerDock);
+  m_layerTreeView->setIndentation(0);
+  m_layerTreeView->setSelectionBehavior(QAbstractItemView::SelectRows);
+  m_layerTreeView->setSelectionMode(QAbstractItemView::SingleSelection);
+  layerDock->setWidget(m_layerTreeView);
+  addDockWidget(Qt::LeftDockWidgetArea, layerDock);
+
+  tabifyDockWidget(moleculeDock, layerDock);
+
   // Switch to our fallback icons if there are no platform-specific icons.
   if (!QIcon::hasThemeIcon("document-new"))
     QIcon::setThemeName("fallback");
@@ -408,6 +427,24 @@ void MainWindow::setupInterface()
   connect(m_moleculeTreeView, &QAbstractItemView::clicked, this,
           &MainWindow::moleculeActivated);
 
+  m_layerModel = new QtGui::LayerModel(this);
+  m_layerTreeView->setModel(m_layerModel);
+  m_layerTreeView->setSelectionBehavior(QAbstractItemView::SelectRows);
+  m_layerTreeView->setAlternatingRowColors(true);
+  m_layerTreeView->header()->setStretchLastSection(false);
+  m_layerTreeView->header()->setVisible(false);
+  m_layerTreeView->header()->setSectionResizeMode(0, QHeaderView::Stretch);
+  for (int i = 1; i < m_layerModel->columnCount(QModelIndex()); ++i) {
+    m_layerTreeView->header()->setSectionResizeMode(i, QHeaderView::Fixed);
+    m_layerTreeView->header()->resizeSection(i, 25);
+  }
+  connect(m_layerTreeView, &QAbstractItemView::activated, this,
+          &MainWindow::layerActivated);
+  connect(m_layerTreeView, &QAbstractItemView::clicked, this,
+          &MainWindow::layerActivated);
+  connect(m_sceneTreeView, &QAbstractItemView::clicked, m_layerModel,
+          &QtGui::LayerModel::updateRows);
+
   viewActivated(glWidget);
   buildTools();
   // Connect to the invalid context signal, check whether GL is initialized.
@@ -426,35 +463,34 @@ void MainWindow::closeEvent(QCloseEvent* e)
   QMainWindow::closeEvent(e);
 }
 
-  void MainWindow::dragEnterEvent(QDragEnterEvent *event)
-  {
-    if (event->mimeData()->hasUrls())
-      event->acceptProposedAction();
-    else
-      event->ignore();
-  }
+void MainWindow::dragEnterEvent(QDragEnterEvent* event)
+{
+  if (event->mimeData()->hasUrls())
+    event->acceptProposedAction();
+  else
+    event->ignore();
+}
 
-  void MainWindow::dropEvent(QDropEvent *event)
-  {
-    if (event->mimeData()->hasUrls()) {
-      // TODO: check for ZIP, TAR, PY scripts (plugins)
-      foreach(const QUrl& url, event->mimeData()->urls() ) {
-        if (url.isLocalFile()) {
-          QString fileName = url.toLocalFile();
-          QFileInfo info(fileName);
-          QString extension = info.completeSuffix(); // e.g. .tar.gz or .pdb.gz
+void MainWindow::dropEvent(QDropEvent* event)
+{
+  if (event->mimeData()->hasUrls()) {
+    // TODO: check for ZIP, TAR, PY scripts (plugins)
+    foreach (const QUrl& url, event->mimeData()->urls()) {
+      if (url.isLocalFile()) {
+        QString fileName = url.toLocalFile();
+        QFileInfo info(fileName);
+        QString extension = info.completeSuffix(); // e.g. .tar.gz or .pdb.gz
 
-          if (extension == "py")
-            addScript(fileName);
-          else
-            openFile(fileName);
-        }
+        if (extension == "py")
+          addScript(fileName);
+        else
+          openFile(fileName);
       }
-      event->acceptProposedAction();
     }
-    else
-      event->ignore();
-  }
+    event->acceptProposedAction();
+  } else
+    event->ignore();
+}
 
 void MainWindow::moleculeReady(int)
 {
@@ -479,16 +515,33 @@ void setWidgetMolecule(T* glWidget, M* mol)
   glWidget->resetCamera();
 }
 
+void setDefaultViews(MultiViewWidget* viewWidget)
+{
+  QSettings settings;
+  // save the enabled scene / render plugins
+  if (GLWidget* glWidget =
+        qobject_cast<GLWidget*>(viewWidget->activeWidget())) {
+
+    const ScenePluginModel* sceneModel = &glWidget->sceneModel();
+    for (auto plugin : sceneModel->scenePlugins()) {
+      QString settingsKey("MainWindow/" + plugin->objectName());
+      bool enabled = settings.value(settingsKey, plugin->isEnabled()).toBool();
+      plugin->setEnabled(enabled);
+    }
+  }
+}
 void MainWindow::setMolecule(Molecule* mol)
 {
   if (!mol)
     return;
 
   // Set the new molecule, ensure both molecules are in the model.
-  if (m_molecule && !m_moleculeModel->molecules().contains(m_molecule))
+  if (m_molecule && !m_moleculeModel->molecules().contains(m_molecule)) {
     m_moleculeModel->addItem(m_molecule);
-  if (!m_moleculeModel->molecules().contains(mol))
+  }
+  if (!m_moleculeModel->molecules().contains(mol)) {
     m_moleculeModel->addItem(mol);
+  }
 
   Molecule* oldMolecule(m_molecule);
   m_molecule = mol;
@@ -507,6 +560,14 @@ void MainWindow::setMolecule(Molecule* mol)
   markMoleculeClean();
   updateWindowTitle();
   m_moleculeModel->setActiveMolecule(m_molecule);
+  m_layerModel->addMolecule(m_molecule);
+  // only set the default values if there is nothing setup
+  // 1 layer (layerCount) and 1º layer + add items
+  if (m_layerModel->layerCount() == 1 && m_layerModel->items() == 2) {
+    setDefaultViews(m_multiViewWidget);
+    refreshDisplayTypes();
+  }
+
   ActiveObjects::instance().setActiveMolecule(m_molecule);
 
   if (oldMolecule)
@@ -533,6 +594,18 @@ void MainWindow::markMoleculeDirty()
   }
 }
 
+void MainWindow::refreshDisplayTypes()
+{
+  m_layerModel->updateRows();
+  // force m_sceneTreeView update (update doesn't update the checkbox)
+  m_sceneTreeView->setFocus();
+  if (m_activeScenePlugin != nullptr) {
+    m_viewDock->setWidget(m_activeScenePlugin->setupWidget());
+  }
+  // reset focus to m_layerTreeView
+  m_layerTreeView->setFocus();
+}
+
 void MainWindow::markMoleculeClean()
 {
   if (m_moleculeDirty) {
@@ -570,8 +643,20 @@ void MainWindow::writeSettings()
   settings.beginGroup("MainWindow");
   settings.setValue("size", size());
   settings.setValue("pos", pos());
+  settings.setValue("perspective", m_viewPerspective->isChecked());
   settings.endGroup();
   settings.setValue("recentFiles", m_recentFiles);
+
+  // save the enabled scene / render plugins
+  if (GLWidget* glWidget =
+        qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget())) {
+
+    const ScenePluginModel* sceneModel = &glWidget->sceneModel();
+    for (auto plugin : sceneModel->scenePlugins()) {
+      QString settingsKey("MainWindow/" + plugin->objectName());
+      settings.setValue(settingsKey, plugin->isEnabled());
+    }
+  }
 }
 
 void MainWindow::readSettings()
@@ -656,8 +741,9 @@ bool MainWindow::addScript(const QString& filePath)
   types << tr("Commands") << tr("Input Generators") << tr("File Formats");
 
   bool ok;
-  QString item = QInputDialog::getItem(this, tr("Install Plugin Script"),
-                                      tr("Script Type:"), types, 0, false, &ok);
+  QString item =
+    QInputDialog::getItem(this, tr("Install Plugin Script"), tr("Script Type:"),
+                          types, 0, false, &ok);
 
   if (!ok || item.isEmpty())
     return false;
@@ -666,7 +752,7 @@ bool MainWindow::addScript(const QString& filePath)
 
   int index = types.indexOf(item);
   // don't translate these
-  switch(index){
+  switch (index) {
     case 0: // commands
       typePath = "commands";
       break;
@@ -682,7 +768,7 @@ bool MainWindow::addScript(const QString& filePath)
 
   QStringList stdPaths =
     QStandardPaths::standardLocations(QStandardPaths::AppLocalDataLocation);
-  
+
   QFileInfo info(filePath);
 
   QString destinationPath(stdPaths[0] + '/' + typePath + '/' + info.fileName());
@@ -854,6 +940,52 @@ void MainWindow::rendererInvalid()
   QTimer::singleShot(500, this, &QWidget::close);
 }
 
+void MainWindow::layerActivated(const QModelIndex& idx)
+{
+  m_layerModel->updateRows();
+  if (idx.row() == m_layerModel->items() - 1) {
+    m_layerModel->addLayer(m_molecule->undoMolecule());
+  } else {
+    bool updateGL = false;
+    if (idx.column() == QtGui::LayerModel::ColumnType::Name) {
+      m_layerModel->setActiveLayer(idx.row(), m_molecule->undoMolecule());
+    } else if (idx.column() == QtGui::LayerModel::ColumnType::Remove) {
+      if (m_layerModel->layerCount() > 1) {
+        m_layerModel->removeItem(idx.row(), m_molecule->undoMolecule());
+        updateGL = true;
+      }
+    } else if (idx.column() == QtGui::LayerModel::ColumnType::Lock) {
+      m_layerModel->flipLocked(idx.row());
+    } else if (idx.column() == QtGui::LayerModel::ColumnType::Visible) {
+      m_layerModel->flipVisible(idx.row());
+      updateGL = true;
+    } else if (idx.column() == QtGui::LayerModel::ColumnType::Menu) {
+      m_layerModel->setActiveLayer(idx.row(), m_molecule->undoMolecule());
+      if (m_sceneDock->isHidden()) {
+        m_sceneDock->show();
+        m_viewDock->show();
+        resizeDocks({ m_sceneDock, m_viewDock }, { 250, 50 }, Qt::Vertical);
+      } else {
+        m_sceneDock->hide();
+        m_viewDock->hide();
+      }
+    }
+
+    if (updateGL) {
+      QWidget* w = m_multiViewWidget->activeWidget();
+      if (GLWidget* glWidget = qobject_cast<QtOpenGL::GLWidget*>(w)) {
+        glWidget->updateScene();
+      }
+#ifdef AVO_USE_VTK
+      else if (vtkGLWidget* vtkWidget = qobject_cast<vtkGLWidget*>(w)) {
+        vtkWidget->updateScene();
+      }
+#endif
+    }
+  }
+  refreshDisplayTypes();
+}
+
 void MainWindow::moleculeActivated(const QModelIndex& idx)
 {
   QObject* obj = static_cast<QObject*>(idx.internalPointer());
@@ -884,22 +1016,25 @@ void MainWindow::sceneItemActivated(const QModelIndex& idx)
   if (!idx.isValid())
     return;
   QObject* obj = static_cast<QObject*>(idx.internalPointer());
-  if (ScenePlugin* scene = qobject_cast<ScenePlugin*>(obj))
+  if (ScenePlugin* scene = qobject_cast<ScenePlugin*>(obj)) {
     m_viewDock->setWidget(scene->setupWidget());
+    m_activeScenePlugin = scene;
+  }
 }
 
 bool populatePluginModel(ScenePluginModel& model, bool editOnly = false)
 {
   if (!model.scenePlugins().empty())
     return false;
+
+  QSettings settings;
   PluginManager* plugin = PluginManager::instance();
   QList<ScenePluginFactory*> scenePluginFactories =
     plugin->pluginFactories<ScenePluginFactory>();
   foreach (ScenePluginFactory* factory, scenePluginFactories) {
     ScenePlugin* scenePlugin = factory->createInstance();
     if (editOnly && scenePlugin) {
-      if (scenePlugin->objectName() == "BallStick" ||
-          scenePlugin->objectName() == "OverlayAxes") {
+      if (scenePlugin->objectName() == "BallStick") {
         model.addItem(scenePlugin);
       } else {
         delete scenePlugin;
@@ -948,15 +1083,13 @@ void MainWindow::viewActivated(QWidget* widget)
       glWidget->updateScene();
     } else {
       m_moleculeModel->setActiveMolecule(glWidget->molecule());
+      m_layerModel->addMolecule(m_molecule);
       // Figure out the active tool - reflect this in the toolbar.
       ToolPlugin* tool = glWidget->activeTool();
       if (tool) {
         QString name = tool->objectName();
         foreach (QAction* action, m_toolToolBar->actions()) {
-          if (action->data().toString() == name)
-            action->setChecked(true);
-          else
-            action->setChecked(false);
+          action->setChecked(action->data().toString() == name);
         }
       }
     }
@@ -965,6 +1098,7 @@ void MainWindow::viewActivated(QWidget* widget)
       m_molecule = glWidget->molecule();
       emit moleculeChanged(m_molecule);
       m_moleculeModel->setActiveMolecule(m_molecule);
+      m_layerModel->addMolecule(m_molecule);
     }
     ActiveObjects::instance().setActiveGLWidget(glWidget);
   }
@@ -978,12 +1112,14 @@ void MainWindow::viewActivated(QWidget* widget)
       vtkWidget->updateScene();
     } else {
       m_moleculeModel->setActiveMolecule(vtkWidget->molecule());
+      m_layerModel->addMolecule(m_molecule);
     }
     if (m_molecule != vtkWidget->molecule() && vtkWidget->molecule()) {
       m_rwMolecule = nullptr;
       m_molecule = vtkWidget->molecule();
       emit moleculeChanged(m_molecule);
       m_moleculeModel->setActiveMolecule(m_molecule);
+      m_layerModel->addMolecule(m_molecule);
     }
   }
 #endif
@@ -1350,6 +1486,7 @@ void MainWindow::undoEdit()
   if (m_molecule) {
     m_molecule->undoMolecule()->undoStack().undo();
     m_molecule->emitChanged(Molecule::Atoms | Molecule::Added);
+    m_layerModel->updateRows();
     activeMoleculeEdited();
   }
 }
@@ -1359,6 +1496,7 @@ void MainWindow::redoEdit()
   if (m_molecule) {
     m_molecule->undoMolecule()->undoStack().redo();
     m_molecule->emitChanged(Molecule::Atoms | Molecule::Added);
+    m_layerModel->updateRows();
     activeMoleculeEdited();
   }
 }
@@ -1625,17 +1763,21 @@ void MainWindow::buildMenu()
   m_menuBuilder->addAction(viewPath, action, 100);
   connect(action, &QAction::triggered, this, &MainWindow::setBackgroundColor);
 
+  // set default projection
+  QSettings settings;
+  bool perspective = settings.value("MainWindow/perspective", true).toBool();
+
   viewPath << tr("Projection");
   m_viewPerspective = new QAction(tr("Perspective"), this);
   m_viewPerspective->setCheckable(true);
-  m_viewPerspective->setChecked(true);
+  m_viewPerspective->setChecked(perspective);
   m_menuBuilder->addAction(viewPath, m_viewPerspective, 10);
   connect(m_viewPerspective, &QAction::triggered, this,
           &MainWindow::setProjectionPerspective);
 
   m_viewOrthographic = new QAction(tr("Orthographic"), this);
   m_viewOrthographic->setCheckable(true);
-  m_viewOrthographic->setChecked(false);
+  m_viewOrthographic->setChecked(!perspective);
   m_menuBuilder->addAction(viewPath, m_viewOrthographic, 10);
   connect(m_viewOrthographic, &QAction::triggered, this,
           &MainWindow::setProjectionOrthographic);
@@ -1645,6 +1787,11 @@ void MainWindow::buildMenu()
   connect(m_viewOrthographic, &QAction::triggered, m_viewPerspective,
           &QAction::toggle);
 
+  if (perspective)
+    setProjectionPerspective();
+  else
+    setProjectionOrthographic();
+
   // Periodic table.
   QStringList extensionsPath;
   extensionsPath << tr("&Extensions");
